@@ -5,7 +5,7 @@ require 'test/unit/testsuite'
 $:.unshift File.expand_path(File.dirname($0))
 $:.unshift 'lib'
 $:.unshift '../lib'
-#require 'coverage'
+$:.unshift 'tests'
 require 'test_json'
 
 class TS_AllTests
