@@ -104,18 +104,22 @@ class TC_JSON < Test::Unit::TestCase
     assert_equal '"\u001f"', 0x1f.chr.to_json
     assert_equal '" "', ' '.to_json
     assert_equal "\"#{0x7f.chr}\"", 0x7f.chr.to_json
-    utf8 = '© ≠ €!'
-    json = '"\u00a9 \u2260 \u20ac!"'
-    assert_equal json, utf8.to_json
-    assert_equal utf8, parse(json)
-    utf8 = "\343\201\202\343\201\204\343\201\206\343\201\210\343\201\212"
-    json = '"\u3042\u3044\u3046\u3048\u304a"'
-    assert_equal json, utf8.to_json
-    assert_equal utf8, parse(json)
-    utf8 = 'საქართველო'
-    json = '"\u10e1\u10d0\u10e5\u10d0\u10e0\u10d7\u10d5\u10d4\u10da\u10dd"'
-    assert_equal json, utf8.to_json
-    assert_equal utf8, parse(json)
+    if JSON.support_unicode?
+      utf8 = '© ≠ €!'
+      json = '"\u00a9 \u2260 \u20ac!"'
+      assert_equal json, utf8.to_json
+      assert_equal utf8, parse(json)
+      utf8 = "\343\201\202\343\201\204\343\201\206\343\201\210\343\201\212"
+      json = '"\u3042\u3044\u3046\u3048\u304a"'
+      assert_equal json, utf8.to_json
+      assert_equal utf8, parse(json)
+      utf8 = 'საქართველო'
+      json = '"\u10e1\u10d0\u10e5\u10d0\u10e0\u10d7\u10d5\u10d4\u10da\u10dd"'
+      assert_equal json, utf8.to_json
+      assert_equal utf8, parse(json)
+    else
+      warn "No unicode support found - skipping test"
+    end
   end
 
   def test_comments
@@ -187,6 +191,10 @@ EOT
   end
 
   def test_utf8_mode
+    unless JSON.support_unicode?
+      warn "No unicode support found - skipping test"
+      return
+    end
     $KCODE = 'NONE'
     utf8 = "© ≠ €! - \001"
     json = "\"© ≠ €! - \\u0001\""
@@ -213,6 +221,10 @@ EOT
     json = '"\\""'
     data = JSON.parse(json)
     assert_equal json, JSON.unparse(data)
+    json = '"\/"'
+    data = JSON.parse(json)
+    assert_equal '/', data
+    assert_equal json, JSON.unparse(data)
   end
 end
   # vim: set et sw=2 ts=2:
